/*
 * Copyright (C) 2013  INRIA.
 *
 * This file is subject to the terms and conditions of the GNU Lesser
 * General Public License v2.1. See the file LICENSE in the top level
 * directory for more details.
 */

/**
 * @defgroup socket_base    Transport layer implementation
 * 
 * @ingroup net
 * @brief   This module implements basic transport layer socket functionality.
 *          This includes 6LoWPAN UDP header compression and (experimental)
 *          6LoWPAN TCP header compression.
 *
 *          It wraps all API types, constants and functions of the transport layer
 *          implementation.
 *
 * @see     <a href="http://tools.ietf.org/html/rfc6282#section-4.3">
 *              RFC 6282 - Compression Format for IPv6 Datagrams over
 *                  IEEE 802.15.4-Based Networks - UDP Header Compression
 *          </a>
 * @see     <a href="http://tools.ietf.org/html/draft-aayadi-6lowpan-tcphc-01">
 *              RFC draft-aayadi-6lowpan-tcphc-01 - TCP header compression for
 *                  6LoWPAN
 *          </a>
 * @{
 * @file
 * @brief   transport_layer functions
 * @author  Oliver Gesch <oliver.gesch@googlemail.com>
 * @author  Martin Lenders <mlenders@inf.fu-berlin.de>
 */

#ifndef SOCKET_BASE_H
#define SOCKET_BASE_H

#include "socket_base/in.h"
#include "socket_base/socket.h"
#include "socket_base/types.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

#endif /* SOCKET_BASE_H */
/** @} */
